{-# LANGUAGE OverloadedStrings #-}

module Main where

import Servant
import Servant.Server
import Servant.Client

import Network.Wai.Handler.Warp
import Network.Wai.Logger

import Network.HTTP.Client hiding (Proxy)

import Control.Monad.Reader

import Data.Text.I18n
import Data.Text.I18n.Po

-- internal imports

import API
import Types
import View
import Control

main :: IO ()
main = do
  manager <- newManager defaultManagerSettings
  (l10n, _) <- getL10n "./locales"
  withStdoutLogger $ \ilog -> do
    let settings = setPort 3000 $ setLogger ilog defaultSettings
        initState = ReadState
          -- { rsManager = manager
          { rsL10n       = l10n
          , rsBackend    = mkClientEnv manager (BaseUrl Http "localhost" 8000 "")
          , rsHashParams = recommendedHashParams
          }
          
    runSettings settings (userApp initState)

userApp initState = serveWithContext userApi (EmptyContext) $
  hoistServerWithContext
    userApi
    Proxy
    (`runReaderT` initState)
    (\mcookie -> userSelectControl mcookie
    :<|> userOverviewControl mcookie
    :<|> userNewControl mcookie
    :<|> userNewPostControl mcookie
    :<|> userManageControl mcookie
    :<|> userManageDetailsSubmitControl mcookie
    :<|> userManageAuthCreateControl mcookie
    :<|> userManageAuthDeleteControl mcookie
    :<|> authControl mcookie
    :<|> authPostControl mcookie
    :<|> authLogoutControl mcookie
    )

userApi :: Proxy UserAPI
userApi = Proxy
